package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickHAT;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickHATActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Set Sleep Mode")
    public void brickHATSetSleepMode(
            @ActionInput(name = "powerOffDelay") long powerOffDelay,
            @ActionInput(name = "powerOffDuration") long powerOffDuration,
            @ActionInput(name = "raspberryPiOff") boolean raspberryPiOff,
            @ActionInput(name = "brickletsOff") boolean brickletsOff,
            @ActionInput(name = "enableSleepIndicator") boolean enableSleepIndicator) throws TinkerforgeException {
        ((BrickHATWrapper)this.handler.getDevice()).setSleepMode(powerOffDelay, powerOffDuration, raspberryPiOff, brickletsOff, enableSleepIndicator);
        
    }

    public static void brickHATSetSleepMode(@Nullable ThingActions actions, long powerOffDelay, long powerOffDuration, boolean raspberryPiOff, boolean brickletsOff, boolean enableSleepIndicator) throws TinkerforgeException {
        if (actions instanceof BrickHATActions) {
            ((BrickHATActions) actions).brickHATSetSleepMode(powerOffDelay, powerOffDuration, raspberryPiOff, brickletsOff, enableSleepIndicator);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATActions class.");
        }
    }

    @RuleAction(label = "Get Sleep Mode")
    public @ActionOutput(name = "powerOffDelay", type="long")
           @ActionOutput(name = "powerOffDuration", type="long")
           @ActionOutput(name = "raspberryPiOff", type="boolean")
           @ActionOutput(name = "brickletsOff", type="boolean")
           @ActionOutput(name = "enableSleepIndicator", type="boolean")
           Map<String, Object> brickHATGetSleepMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickHAT.SleepMode value = ((BrickHATWrapper)this.handler.getDevice()).getSleepMode();
        
        result.put("powerOffDelay", value.powerOffDelay);
        result.put("powerOffDuration", value.powerOffDuration);
        result.put("raspberryPiOff", value.raspberryPiOff);
        result.put("brickletsOff", value.brickletsOff);
        result.put("enableSleepIndicator", value.enableSleepIndicator);
        return result;
    }

    public static Map<String, Object> brickHATGetSleepMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickHATActions) {
            return ((BrickHATActions) actions).brickHATGetSleepMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATActions class.");
        }
    }

    @RuleAction(label = "Set Bricklet Power")
    public void brickHATSetBrickletPower(
            @ActionInput(name = "brickletPower") boolean brickletPower) throws TinkerforgeException {
        ((BrickHATWrapper)this.handler.getDevice()).setBrickletPower(brickletPower);
        if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickHATPowerBricklets")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickHATPowerBricklets"), RefreshType.REFRESH);
}
    }

    public static void brickHATSetBrickletPower(@Nullable ThingActions actions, boolean brickletPower) throws TinkerforgeException {
        if (actions instanceof BrickHATActions) {
            ((BrickHATActions) actions).brickHATSetBrickletPower(brickletPower);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATActions class.");
        }
    }

    @RuleAction(label = "Get Bricklet Power")
    public @ActionOutput(name = "brickletPower", type="boolean")
           Map<String, Object> brickHATGetBrickletPower(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickHATWrapper)this.handler.getDevice()).getBrickletPower();
        
        result.put("brickletPower", value);
        return result;
    }

    public static Map<String, Object> brickHATGetBrickletPower(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickHATActions) {
            return ((BrickHATActions) actions).brickHATGetBrickletPower();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATActions class.");
        }
    }

    @RuleAction(label = "Get Voltages")
    public @ActionOutput(name = "voltageUSB", type="int")
           @ActionOutput(name = "voltageDC", type="int")
           Map<String, Object> brickHATGetVoltages(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickHAT.Voltages value = ((BrickHATWrapper)this.handler.getDevice()).getVoltages();
        
        result.put("voltageUSB", value.voltageUSB);
        result.put("voltageDC", value.voltageDC);
        return result;
    }

    public static Map<String, Object> brickHATGetVoltages(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickHATActions) {
            return ((BrickHATActions) actions).brickHATGetVoltages();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATActions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickHATReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickHATWrapper)this.handler.getDevice()).readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickHATReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickHATActions) {
            return ((BrickHATActions) actions).brickHATReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATActions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickHATGetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickHATWrapper)this.handler.getDevice()).getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickHATGetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickHATActions) {
            return ((BrickHATActions) actions).brickHATGetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATActions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickHATGetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickHATWrapper)this.handler.getDevice()).getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickHATGetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickHATActions) {
            return ((BrickHATActions) actions).brickHATGetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATActions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickHATGetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickHATWrapper)this.handler.getDevice()).getBootloaderMode();
        
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickHATGetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickHATActions) {
            return ((BrickHATActions) actions).brickHATGetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickHATGetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickHAT.SPITFPErrorCount value = ((BrickHATWrapper)this.handler.getDevice()).getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickHATGetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickHATActions) {
            return ((BrickHATActions) actions).brickHATGetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickHATGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickHAT.Identity value = ((BrickHATWrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickHATGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickHATActions) {
            return ((BrickHATActions) actions).brickHATGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickHATActions class.");
        }
    }
}
