package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletAnalogOut;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletAnalogOutActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Set Voltage")
    public void brickletAnalogOutSetVoltage(
            @ActionInput(name = "voltage") int voltage) throws TinkerforgeException {
        ((BrickletAnalogOutWrapper)this.handler.getDevice()).setVoltage(voltage);
        if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletAnalogOutVoltage")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletAnalogOutVoltage"), RefreshType.REFRESH);
}
    }

    public static void brickletAnalogOutSetVoltage(@Nullable ThingActions actions, int voltage) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutActions) {
            ((BrickletAnalogOutActions) actions).brickletAnalogOutSetVoltage(voltage);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutActions class.");
        }
    }

    @RuleAction(label = "Get Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickletAnalogOutGetVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAnalogOutWrapper)this.handler.getDevice()).getVoltage();
        
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutGetVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutActions) {
            return ((BrickletAnalogOutActions) actions).brickletAnalogOutGetVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutActions class.");
        }
    }

    @RuleAction(label = "Set Mode")
    public void brickletAnalogOutSetMode(
            @ActionInput(name = "mode") short mode) throws TinkerforgeException {
        ((BrickletAnalogOutWrapper)this.handler.getDevice()).setMode(mode);
        if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletAnalogOutMode")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletAnalogOutMode"), RefreshType.REFRESH);
}
    }

    public static void brickletAnalogOutSetMode(@Nullable ThingActions actions, short mode) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutActions) {
            ((BrickletAnalogOutActions) actions).brickletAnalogOutSetMode(mode);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutActions class.");
        }
    }

    @RuleAction(label = "Get Mode")
    public @ActionOutput(name = "mode", type="short")
           Map<String, Object> brickletAnalogOutGetMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickletAnalogOutWrapper)this.handler.getDevice()).getMode();
        
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutGetMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutActions) {
            return ((BrickletAnalogOutActions) actions).brickletAnalogOutGetMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletAnalogOutGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletAnalogOut.Identity value = ((BrickletAnalogOutWrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutActions) {
            return ((BrickletAnalogOutActions) actions).brickletAnalogOutGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutActions class.");
        }
    }
}
