package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletEnergyMonitor;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletEnergyMonitorActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Energy Data")
    public @ActionOutput(name = "voltage", type="int")
           @ActionOutput(name = "current", type="int")
           @ActionOutput(name = "energy", type="int")
           @ActionOutput(name = "realPower", type="int")
           @ActionOutput(name = "apparentPower", type="int")
           @ActionOutput(name = "reactivePower", type="int")
           @ActionOutput(name = "powerFactor", type="int")
           @ActionOutput(name = "frequency", type="int")
           Map<String, Object> brickletEnergyMonitorGetEnergyData(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletEnergyMonitor.EnergyData value = ((BrickletEnergyMonitorWrapper)this.handler.getDevice()).getEnergyData();
        
        result.put("voltage", value.voltage);
        result.put("current", value.current);
        result.put("energy", value.energy);
        result.put("realPower", value.realPower);
        result.put("apparentPower", value.apparentPower);
        result.put("reactivePower", value.reactivePower);
        result.put("powerFactor", value.powerFactor);
        result.put("frequency", value.frequency);
        return result;
    }

    public static Map<String, Object> brickletEnergyMonitorGetEnergyData(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletEnergyMonitorActions) {
            return ((BrickletEnergyMonitorActions) actions).brickletEnergyMonitorGetEnergyData();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletEnergyMonitorActions class.");
        }
    }

    @RuleAction(label = "Reset Energy")
    public void brickletEnergyMonitorResetEnergy(
            ) throws TinkerforgeException {
        ((BrickletEnergyMonitorWrapper)this.handler.getDevice()).resetEnergy();
        
    }

    public static void brickletEnergyMonitorResetEnergy(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletEnergyMonitorActions) {
            ((BrickletEnergyMonitorActions) actions).brickletEnergyMonitorResetEnergy();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletEnergyMonitorActions class.");
        }
    }

    @RuleAction(label = "Get Waveform")
    public @ActionOutput(name = "waveform", type="int[]")
           Map<String, Object> brickletEnergyMonitorGetWaveform(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int[] value = ((BrickletEnergyMonitorWrapper)this.handler.getDevice()).getWaveform();
        
        result.put("waveform", value);
        return result;
    }

    public static Map<String, Object> brickletEnergyMonitorGetWaveform(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletEnergyMonitorActions) {
            return ((BrickletEnergyMonitorActions) actions).brickletEnergyMonitorGetWaveform();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletEnergyMonitorActions class.");
        }
    }

    @RuleAction(label = "Get Transformer Status")
    public @ActionOutput(name = "voltageTransformerConnected", type="boolean")
           @ActionOutput(name = "currentTransformerConnected", type="boolean")
           Map<String, Object> brickletEnergyMonitorGetTransformerStatus(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletEnergyMonitor.TransformerStatus value = ((BrickletEnergyMonitorWrapper)this.handler.getDevice()).getTransformerStatus();
        
        result.put("voltageTransformerConnected", value.voltageTransformerConnected);
        result.put("currentTransformerConnected", value.currentTransformerConnected);
        return result;
    }

    public static Map<String, Object> brickletEnergyMonitorGetTransformerStatus(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletEnergyMonitorActions) {
            return ((BrickletEnergyMonitorActions) actions).brickletEnergyMonitorGetTransformerStatus();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletEnergyMonitorActions class.");
        }
    }

    @RuleAction(label = "Get Transformer Calibration")
    public @ActionOutput(name = "voltageRatio", type="int")
           @ActionOutput(name = "currentRatio", type="int")
           @ActionOutput(name = "phaseShift", type="int")
           Map<String, Object> brickletEnergyMonitorGetTransformerCalibration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletEnergyMonitor.TransformerCalibration value = ((BrickletEnergyMonitorWrapper)this.handler.getDevice()).getTransformerCalibration();
        
        result.put("voltageRatio", value.voltageRatio);
        result.put("currentRatio", value.currentRatio);
        result.put("phaseShift", value.phaseShift);
        return result;
    }

    public static Map<String, Object> brickletEnergyMonitorGetTransformerCalibration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletEnergyMonitorActions) {
            return ((BrickletEnergyMonitorActions) actions).brickletEnergyMonitorGetTransformerCalibration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletEnergyMonitorActions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletEnergyMonitorReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletEnergyMonitorWrapper)this.handler.getDevice()).readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletEnergyMonitorReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletEnergyMonitorActions) {
            return ((BrickletEnergyMonitorActions) actions).brickletEnergyMonitorReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletEnergyMonitorActions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletEnergyMonitorGetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletEnergyMonitorWrapper)this.handler.getDevice()).getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletEnergyMonitorGetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletEnergyMonitorActions) {
            return ((BrickletEnergyMonitorActions) actions).brickletEnergyMonitorGetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletEnergyMonitorActions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletEnergyMonitorGetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletEnergyMonitorWrapper)this.handler.getDevice()).getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletEnergyMonitorGetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletEnergyMonitorActions) {
            return ((BrickletEnergyMonitorActions) actions).brickletEnergyMonitorGetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletEnergyMonitorActions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletEnergyMonitorGetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletEnergyMonitorWrapper)this.handler.getDevice()).getBootloaderMode();
        
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletEnergyMonitorGetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletEnergyMonitorActions) {
            return ((BrickletEnergyMonitorActions) actions).brickletEnergyMonitorGetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletEnergyMonitorActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletEnergyMonitorGetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletEnergyMonitor.SPITFPErrorCount value = ((BrickletEnergyMonitorWrapper)this.handler.getDevice()).getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletEnergyMonitorGetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletEnergyMonitorActions) {
            return ((BrickletEnergyMonitorActions) actions).brickletEnergyMonitorGetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletEnergyMonitorActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletEnergyMonitorGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletEnergyMonitor.Identity value = ((BrickletEnergyMonitorWrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletEnergyMonitorGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletEnergyMonitorActions) {
            return ((BrickletEnergyMonitorActions) actions).brickletEnergyMonitorGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletEnergyMonitorActions class.");
        }
    }
}
