package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletIO4;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIO4Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Set Value")
    public void brickletIO4SetValue(
            @ActionInput(name = "valueMask") short valueMask) throws TinkerforgeException {
        ((BrickletIO4Wrapper)this.handler.getDevice()).setValue(valueMask);
        if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4OutputPin0")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4OutputPin0"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4OutputPin1")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4OutputPin1"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4OutputPin2")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4OutputPin2"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4OutputPin3")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4OutputPin3"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4MonoflopPin0")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4MonoflopPin0"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4MonoflopPin1")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4MonoflopPin1"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4MonoflopPin2")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4MonoflopPin2"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4MonoflopPin3")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4MonoflopPin3"), RefreshType.REFRESH);
}
    }

    public static void brickletIO4SetValue(@Nullable ThingActions actions, short valueMask) throws TinkerforgeException {
        if (actions instanceof BrickletIO4Actions) {
            ((BrickletIO4Actions) actions).brickletIO4SetValue(valueMask);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO4Actions class.");
        }
    }

    @RuleAction(label = "Set Selected Values")
    public void brickletIO4SetSelectedValues(
            @ActionInput(name = "selectionMask") short selectionMask,
            @ActionInput(name = "valueMask") short valueMask) throws TinkerforgeException {
        ((BrickletIO4Wrapper)this.handler.getDevice()).setSelectedValues(selectionMask, valueMask);
        if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4OutputPin0")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4OutputPin0"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4OutputPin1")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4OutputPin1"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4OutputPin2")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4OutputPin2"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4OutputPin3")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4OutputPin3"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4MonoflopPin0")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4MonoflopPin0"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4MonoflopPin1")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4MonoflopPin1"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4MonoflopPin2")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4MonoflopPin2"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4MonoflopPin3")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4MonoflopPin3"), RefreshType.REFRESH);
}
    }

    public static void brickletIO4SetSelectedValues(@Nullable ThingActions actions, short selectionMask, short valueMask) throws TinkerforgeException {
        if (actions instanceof BrickletIO4Actions) {
            ((BrickletIO4Actions) actions).brickletIO4SetSelectedValues(selectionMask, valueMask);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO4Actions class.");
        }
    }

    @RuleAction(label = "Set Monoflop")
    public void brickletIO4SetMonoflop(
            @ActionInput(name = "selectionMask") short selectionMask,
            @ActionInput(name = "valueMask") short valueMask,
            @ActionInput(name = "time") long time) throws TinkerforgeException {
        ((BrickletIO4Wrapper)this.handler.getDevice()).setMonoflop(selectionMask, valueMask, time);
        if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4OutputPin0")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4OutputPin0"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4OutputPin1")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4OutputPin1"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4OutputPin2")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4OutputPin2"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4OutputPin3")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4OutputPin3"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4MonoflopPin0")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4MonoflopPin0"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4MonoflopPin1")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4MonoflopPin1"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4MonoflopPin2")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4MonoflopPin2"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4MonoflopPin3")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4MonoflopPin3"), RefreshType.REFRESH);
}
    }

    public static void brickletIO4SetMonoflop(@Nullable ThingActions actions, short selectionMask, short valueMask, long time) throws TinkerforgeException {
        if (actions instanceof BrickletIO4Actions) {
            ((BrickletIO4Actions) actions).brickletIO4SetMonoflop(selectionMask, valueMask, time);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO4Actions class.");
        }
    }

    @RuleAction(label = "Get Value")
    public @ActionOutput(name = "valueMask", type="short")
           Map<String, Object> brickletIO4GetValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickletIO4Wrapper)this.handler.getDevice()).getValue();
        
        result.put("valueMask", value);
        return result;
    }

    public static Map<String, Object> brickletIO4GetValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIO4Actions) {
            return ((BrickletIO4Actions) actions).brickletIO4GetValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO4Actions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "directionMask", type="short")
           @ActionOutput(name = "valueMask", type="short")
           Map<String, Object> brickletIO4GetConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIO4.Configuration value = ((BrickletIO4Wrapper)this.handler.getDevice()).getConfiguration();
        
        result.put("directionMask", value.directionMask);
        result.put("valueMask", value.valueMask);
        return result;
    }

    public static Map<String, Object> brickletIO4GetConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIO4Actions) {
            return ((BrickletIO4Actions) actions).brickletIO4GetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO4Actions class.");
        }
    }

    @RuleAction(label = "Get Edge Count")
    public @ActionOutput(name = "count", type="long")
           Map<String, Object> brickletIO4GetEdgeCount(
            @ActionInput(name = "pin") short pin,
            @ActionInput(name = "resetCounter") boolean resetCounter) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletIO4Wrapper)this.handler.getDevice()).getEdgeCount(pin, resetCounter);
        if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4EdgeCountPin0")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4EdgeCountPin0"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4EdgeCountPin1")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4EdgeCountPin1"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4EdgeCountPin2")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4EdgeCountPin2"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIO4EdgeCountPin3")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIO4EdgeCountPin3"), RefreshType.REFRESH);
}
        result.put("count", value);
        return result;
    }

    public static Map<String, Object> brickletIO4GetEdgeCount(@Nullable ThingActions actions, short pin, boolean resetCounter) throws TinkerforgeException {
        if (actions instanceof BrickletIO4Actions) {
            return ((BrickletIO4Actions) actions).brickletIO4GetEdgeCount(pin, resetCounter);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO4Actions class.");
        }
    }

    @RuleAction(label = "Get Monoflop")
    public @ActionOutput(name = "value", type="short")
           @ActionOutput(name = "time", type="long")
           @ActionOutput(name = "timeRemaining", type="long")
           Map<String, Object> brickletIO4GetMonoflop(
            @ActionInput(name = "pin") short pin) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIO4.Monoflop value = ((BrickletIO4Wrapper)this.handler.getDevice()).getMonoflop(pin);
        
        result.put("value", value.value);
        result.put("time", value.time);
        result.put("timeRemaining", value.timeRemaining);
        return result;
    }

    public static Map<String, Object> brickletIO4GetMonoflop(@Nullable ThingActions actions, short pin) throws TinkerforgeException {
        if (actions instanceof BrickletIO4Actions) {
            return ((BrickletIO4Actions) actions).brickletIO4GetMonoflop(pin);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO4Actions class.");
        }
    }

    @RuleAction(label = "Get Edge Count Config")
    public @ActionOutput(name = "edgeType", type="short")
           @ActionOutput(name = "debounce", type="short")
           Map<String, Object> brickletIO4GetEdgeCountConfig(
            @ActionInput(name = "pin") short pin) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIO4.EdgeCountConfig value = ((BrickletIO4Wrapper)this.handler.getDevice()).getEdgeCountConfig(pin);
        
        result.put("edgeType", value.edgeType);
        result.put("debounce", value.debounce);
        return result;
    }

    public static Map<String, Object> brickletIO4GetEdgeCountConfig(@Nullable ThingActions actions, short pin) throws TinkerforgeException {
        if (actions instanceof BrickletIO4Actions) {
            return ((BrickletIO4Actions) actions).brickletIO4GetEdgeCountConfig(pin);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO4Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletIO4GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIO4.Identity value = ((BrickletIO4Wrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletIO4GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIO4Actions) {
            return ((BrickletIO4Actions) actions).brickletIO4GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO4Actions class.");
        }
    }
}
