package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletIndustrialDigitalIn4;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIndustrialDigitalIn4Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Value")
    public @ActionOutput(name = "valueMask", type="int")
           Map<String, Object> brickletIndustrialDigitalIn4GetValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialDigitalIn4Wrapper)this.handler.getDevice()).getValue();
        
        result.put("valueMask", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalIn4GetValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDigitalIn4Actions) {
            return ((BrickletIndustrialDigitalIn4Actions) actions).brickletIndustrialDigitalIn4GetValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDigitalIn4Actions class.");
        }
    }

    @RuleAction(label = "Get Edge Count")
    public @ActionOutput(name = "count", type="long")
           Map<String, Object> brickletIndustrialDigitalIn4GetEdgeCount(
            @ActionInput(name = "pin") short pin,
            @ActionInput(name = "resetCounter") boolean resetCounter) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletIndustrialDigitalIn4Wrapper)this.handler.getDevice()).getEdgeCount(pin, resetCounter);
        if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialDigitalIn4EdgeCountPin0")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialDigitalIn4EdgeCountPin0"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialDigitalIn4EdgeCountPin1")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialDigitalIn4EdgeCountPin1"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialDigitalIn4EdgeCountPin2")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialDigitalIn4EdgeCountPin2"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialDigitalIn4EdgeCountPin3")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletIndustrialDigitalIn4EdgeCountPin3"), RefreshType.REFRESH);
}
        result.put("count", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalIn4GetEdgeCount(@Nullable ThingActions actions, short pin, boolean resetCounter) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDigitalIn4Actions) {
            return ((BrickletIndustrialDigitalIn4Actions) actions).brickletIndustrialDigitalIn4GetEdgeCount(pin, resetCounter);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDigitalIn4Actions class.");
        }
    }

    @RuleAction(label = "Get Edge Count Config")
    public @ActionOutput(name = "edgeType", type="short")
           @ActionOutput(name = "debounce", type="short")
           Map<String, Object> brickletIndustrialDigitalIn4GetEdgeCountConfig(
            @ActionInput(name = "pin") short pin) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialDigitalIn4.EdgeCountConfig value = ((BrickletIndustrialDigitalIn4Wrapper)this.handler.getDevice()).getEdgeCountConfig(pin);
        
        result.put("edgeType", value.edgeType);
        result.put("debounce", value.debounce);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalIn4GetEdgeCountConfig(@Nullable ThingActions actions, short pin) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDigitalIn4Actions) {
            return ((BrickletIndustrialDigitalIn4Actions) actions).brickletIndustrialDigitalIn4GetEdgeCountConfig(pin);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDigitalIn4Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletIndustrialDigitalIn4GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialDigitalIn4.Identity value = ((BrickletIndustrialDigitalIn4Wrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalIn4GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDigitalIn4Actions) {
            return ((BrickletIndustrialDigitalIn4Actions) actions).brickletIndustrialDigitalIn4GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDigitalIn4Actions class.");
        }
    }
}
