package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletIndustrialDualAnalogIn;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIndustrialDualAnalogInActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickletIndustrialDualAnalogInGetVoltage(
            @ActionInput(name = "channel") short channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialDualAnalogInWrapper)this.handler.getDevice()).getVoltage(channel);
        
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDualAnalogInGetVoltage(@Nullable ThingActions actions, short channel) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInActions) {
            return ((BrickletIndustrialDualAnalogInActions) actions).brickletIndustrialDualAnalogInGetVoltage(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInActions class.");
        }
    }

    @RuleAction(label = "Get Sample Rate")
    public @ActionOutput(name = "rate", type="short")
           Map<String, Object> brickletIndustrialDualAnalogInGetSampleRate(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickletIndustrialDualAnalogInWrapper)this.handler.getDevice()).getSampleRate();
        
        result.put("rate", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDualAnalogInGetSampleRate(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInActions) {
            return ((BrickletIndustrialDualAnalogInActions) actions).brickletIndustrialDualAnalogInGetSampleRate();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInActions class.");
        }
    }

    @RuleAction(label = "Get Calibration")
    public @ActionOutput(name = "offset", type="int[]")
           @ActionOutput(name = "gain", type="int[]")
           Map<String, Object> brickletIndustrialDualAnalogInGetCalibration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialDualAnalogIn.Calibration value = ((BrickletIndustrialDualAnalogInWrapper)this.handler.getDevice()).getCalibration();
        
        result.put("offset", value.offset);
        result.put("gain", value.gain);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDualAnalogInGetCalibration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInActions) {
            return ((BrickletIndustrialDualAnalogInActions) actions).brickletIndustrialDualAnalogInGetCalibration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInActions class.");
        }
    }

    @RuleAction(label = "Get ADC Values")
    public @ActionOutput(name = "value", type="int[]")
           Map<String, Object> brickletIndustrialDualAnalogInGetADCValues(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int[] value = ((BrickletIndustrialDualAnalogInWrapper)this.handler.getDevice()).getADCValues();
        
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDualAnalogInGetADCValues(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInActions) {
            return ((BrickletIndustrialDualAnalogInActions) actions).brickletIndustrialDualAnalogInGetADCValues();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletIndustrialDualAnalogInGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialDualAnalogIn.Identity value = ((BrickletIndustrialDualAnalogInWrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDualAnalogInGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInActions) {
            return ((BrickletIndustrialDualAnalogInActions) actions).brickletIndustrialDualAnalogInGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInActions class.");
        }
    }
}
