package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletMultiTouch;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletMultiTouchActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Touch State")
    public @ActionOutput(name = "state", type="int")
           Map<String, Object> brickletMultiTouchGetTouchState(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletMultiTouchWrapper)this.handler.getDevice()).getTouchState();
        
        result.put("state", value);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchGetTouchState(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchActions) {
            return ((BrickletMultiTouchActions) actions).brickletMultiTouchGetTouchState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchActions class.");
        }
    }

    @RuleAction(label = "Recalibrate")
    public void brickletMultiTouchRecalibrate(
            ) throws TinkerforgeException {
        ((BrickletMultiTouchWrapper)this.handler.getDevice()).recalibrate();
        
    }

    public static void brickletMultiTouchRecalibrate(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchActions) {
            ((BrickletMultiTouchActions) actions).brickletMultiTouchRecalibrate();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchActions class.");
        }
    }

    @RuleAction(label = "Get Electrode Config")
    public @ActionOutput(name = "enabledElectrodes", type="int")
           Map<String, Object> brickletMultiTouchGetElectrodeConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletMultiTouchWrapper)this.handler.getDevice()).getElectrodeConfig();
        
        result.put("enabledElectrodes", value);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchGetElectrodeConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchActions) {
            return ((BrickletMultiTouchActions) actions).brickletMultiTouchGetElectrodeConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchActions class.");
        }
    }

    @RuleAction(label = "Get Electrode Sensitivity")
    public @ActionOutput(name = "sensitivity", type="short")
           Map<String, Object> brickletMultiTouchGetElectrodeSensitivity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickletMultiTouchWrapper)this.handler.getDevice()).getElectrodeSensitivity();
        
        result.put("sensitivity", value);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchGetElectrodeSensitivity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchActions) {
            return ((BrickletMultiTouchActions) actions).brickletMultiTouchGetElectrodeSensitivity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletMultiTouchGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletMultiTouch.Identity value = ((BrickletMultiTouchWrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletMultiTouchGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchActions) {
            return ((BrickletMultiTouchActions) actions).brickletMultiTouchGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchActions class.");
        }
    }
}
