package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletNFCRFID;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletNFCRFIDActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Request Tag ID")
    public void brickletNFCRFIDRequestTagID(
            @ActionInput(name = "tagType") short tagType) throws TinkerforgeException {
        ((BrickletNFCRFIDWrapper)this.handler.getDevice()).requestTagID(tagType);
        
    }

    public static void brickletNFCRFIDRequestTagID(@Nullable ThingActions actions, short tagType) throws TinkerforgeException {
        if (actions instanceof BrickletNFCRFIDActions) {
            ((BrickletNFCRFIDActions) actions).brickletNFCRFIDRequestTagID(tagType);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletNFCRFIDActions class.");
        }
    }

    @RuleAction(label = "Get Tag ID")
    public @ActionOutput(name = "tagType", type="short")
           @ActionOutput(name = "tidLength", type="short")
           @ActionOutput(name = "tid", type="short[]")
           Map<String, Object> brickletNFCRFIDGetTagID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletNFCRFID.TagID value = ((BrickletNFCRFIDWrapper)this.handler.getDevice()).getTagID();
        
        result.put("tagType", value.tagType);
        result.put("tidLength", value.tidLength);
        result.put("tid", value.tid);
        return result;
    }

    public static Map<String, Object> brickletNFCRFIDGetTagID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletNFCRFIDActions) {
            return ((BrickletNFCRFIDActions) actions).brickletNFCRFIDGetTagID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletNFCRFIDActions class.");
        }
    }

    @RuleAction(label = "Get State")
    public @ActionOutput(name = "state", type="short")
           @ActionOutput(name = "idle", type="boolean")
           Map<String, Object> brickletNFCRFIDGetState(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletNFCRFID.State value = ((BrickletNFCRFIDWrapper)this.handler.getDevice()).getState();
        
        result.put("state", value.state);
        result.put("idle", value.idle);
        return result;
    }

    public static Map<String, Object> brickletNFCRFIDGetState(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletNFCRFIDActions) {
            return ((BrickletNFCRFIDActions) actions).brickletNFCRFIDGetState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletNFCRFIDActions class.");
        }
    }

    @RuleAction(label = "Authenticate Mifare Classic Page")
    public void brickletNFCRFIDAuthenticateMifareClassicPage(
            @ActionInput(name = "page") int page,
            @ActionInput(name = "keyNumber") short keyNumber,
            @ActionInput(name = "key") short[] key) throws TinkerforgeException {
        ((BrickletNFCRFIDWrapper)this.handler.getDevice()).authenticateMifareClassicPage(page, keyNumber, key);
        
    }

    public static void brickletNFCRFIDAuthenticateMifareClassicPage(@Nullable ThingActions actions, int page, short keyNumber, short[] key) throws TinkerforgeException {
        if (actions instanceof BrickletNFCRFIDActions) {
            ((BrickletNFCRFIDActions) actions).brickletNFCRFIDAuthenticateMifareClassicPage(page, keyNumber, key);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletNFCRFIDActions class.");
        }
    }

    @RuleAction(label = "Write Page")
    public void brickletNFCRFIDWritePage(
            @ActionInput(name = "page") int page,
            @ActionInput(name = "data") short[] data) throws TinkerforgeException {
        ((BrickletNFCRFIDWrapper)this.handler.getDevice()).writePage(page, data);
        
    }

    public static void brickletNFCRFIDWritePage(@Nullable ThingActions actions, int page, short[] data) throws TinkerforgeException {
        if (actions instanceof BrickletNFCRFIDActions) {
            ((BrickletNFCRFIDActions) actions).brickletNFCRFIDWritePage(page, data);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletNFCRFIDActions class.");
        }
    }

    @RuleAction(label = "Request Page")
    public void brickletNFCRFIDRequestPage(
            @ActionInput(name = "page") int page) throws TinkerforgeException {
        ((BrickletNFCRFIDWrapper)this.handler.getDevice()).requestPage(page);
        
    }

    public static void brickletNFCRFIDRequestPage(@Nullable ThingActions actions, int page) throws TinkerforgeException {
        if (actions instanceof BrickletNFCRFIDActions) {
            ((BrickletNFCRFIDActions) actions).brickletNFCRFIDRequestPage(page);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletNFCRFIDActions class.");
        }
    }

    @RuleAction(label = "Get Page")
    public @ActionOutput(name = "data", type="short[]")
           Map<String, Object> brickletNFCRFIDGetPage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short[] value = ((BrickletNFCRFIDWrapper)this.handler.getDevice()).getPage();
        
        result.put("data", value);
        return result;
    }

    public static Map<String, Object> brickletNFCRFIDGetPage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletNFCRFIDActions) {
            return ((BrickletNFCRFIDActions) actions).brickletNFCRFIDGetPage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletNFCRFIDActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletNFCRFIDGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletNFCRFID.Identity value = ((BrickletNFCRFIDWrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletNFCRFIDGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletNFCRFIDActions) {
            return ((BrickletNFCRFIDActions) actions).brickletNFCRFIDGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletNFCRFIDActions class.");
        }
    }
}
