package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletOLED128x64V2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletOLED128x64V2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Write Pixels")
    public void brickletOLED128x64V2WritePixels(
            @ActionInput(name = "xStart") int xStart,
            @ActionInput(name = "yStart") int yStart,
            @ActionInput(name = "xEnd") int xEnd,
            @ActionInput(name = "yEnd") int yEnd,
            @ActionInput(name = "pixels") boolean[] pixels) throws TinkerforgeException {
        ((BrickletOLED128x64V2Wrapper)this.handler.getDevice()).writePixels(xStart, yStart, xEnd, yEnd, pixels);
        
    }

    public static void brickletOLED128x64V2WritePixels(@Nullable ThingActions actions, int xStart, int yStart, int xEnd, int yEnd, boolean[] pixels) throws TinkerforgeException {
        if (actions instanceof BrickletOLED128x64V2Actions) {
            ((BrickletOLED128x64V2Actions) actions).brickletOLED128x64V2WritePixels(xStart, yStart, xEnd, yEnd, pixels);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED128x64V2Actions class.");
        }
    }

    @RuleAction(label = "Read Pixels")
    public @ActionOutput(name = "pixels", type="boolean[]")
           Map<String, Object> brickletOLED128x64V2ReadPixels(
            @ActionInput(name = "xStart") int xStart,
            @ActionInput(name = "yStart") int yStart,
            @ActionInput(name = "xEnd") int xEnd,
            @ActionInput(name = "yEnd") int yEnd) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean[] value = ((BrickletOLED128x64V2Wrapper)this.handler.getDevice()).readPixels(xStart, yStart, xEnd, yEnd);
        
        result.put("pixels", value);
        return result;
    }

    public static Map<String, Object> brickletOLED128x64V2ReadPixels(@Nullable ThingActions actions, int xStart, int yStart, int xEnd, int yEnd) throws TinkerforgeException {
        if (actions instanceof BrickletOLED128x64V2Actions) {
            return ((BrickletOLED128x64V2Actions) actions).brickletOLED128x64V2ReadPixels(xStart, yStart, xEnd, yEnd);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED128x64V2Actions class.");
        }
    }

    @RuleAction(label = "Clear Display")
    public void brickletOLED128x64V2ClearDisplay(
            ) throws TinkerforgeException {
        ((BrickletOLED128x64V2Wrapper)this.handler.getDevice()).clearDisplay();
        
    }

    public static void brickletOLED128x64V2ClearDisplay(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOLED128x64V2Actions) {
            ((BrickletOLED128x64V2Actions) actions).brickletOLED128x64V2ClearDisplay();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED128x64V2Actions class.");
        }
    }

    @RuleAction(label = "Draw Buffered Frame")
    public void brickletOLED128x64V2DrawBufferedFrame(
            @ActionInput(name = "forceCompleteRedraw") boolean forceCompleteRedraw) throws TinkerforgeException {
        ((BrickletOLED128x64V2Wrapper)this.handler.getDevice()).drawBufferedFrame(forceCompleteRedraw);
        
    }

    public static void brickletOLED128x64V2DrawBufferedFrame(@Nullable ThingActions actions, boolean forceCompleteRedraw) throws TinkerforgeException {
        if (actions instanceof BrickletOLED128x64V2Actions) {
            ((BrickletOLED128x64V2Actions) actions).brickletOLED128x64V2DrawBufferedFrame(forceCompleteRedraw);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED128x64V2Actions class.");
        }
    }

    @RuleAction(label = "Write Line")
    public void brickletOLED128x64V2WriteLine(
            @ActionInput(name = "line") int line,
            @ActionInput(name = "position") int position,
            @ActionInput(name = "text") String text) throws TinkerforgeException {
        ((BrickletOLED128x64V2Wrapper)this.handler.getDevice()).writeLine(line, position, text);
        
    }

    public static void brickletOLED128x64V2WriteLine(@Nullable ThingActions actions, int line, int position, String text) throws TinkerforgeException {
        if (actions instanceof BrickletOLED128x64V2Actions) {
            ((BrickletOLED128x64V2Actions) actions).brickletOLED128x64V2WriteLine(line, position, text);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED128x64V2Actions class.");
        }
    }

    @RuleAction(label = "Get Display Configuration")
    public @ActionOutput(name = "contrast", type="int")
           @ActionOutput(name = "invert", type="boolean")
           @ActionOutput(name = "automaticDraw", type="boolean")
           Map<String, Object> brickletOLED128x64V2GetDisplayConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletOLED128x64V2.DisplayConfiguration value = ((BrickletOLED128x64V2Wrapper)this.handler.getDevice()).getDisplayConfiguration();
        
        result.put("contrast", value.contrast);
        result.put("invert", value.invert);
        result.put("automaticDraw", value.automaticDraw);
        return result;
    }

    public static Map<String, Object> brickletOLED128x64V2GetDisplayConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOLED128x64V2Actions) {
            return ((BrickletOLED128x64V2Actions) actions).brickletOLED128x64V2GetDisplayConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED128x64V2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletOLED128x64V2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletOLED128x64V2Wrapper)this.handler.getDevice()).readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletOLED128x64V2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOLED128x64V2Actions) {
            return ((BrickletOLED128x64V2Actions) actions).brickletOLED128x64V2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED128x64V2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletOLED128x64V2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletOLED128x64V2Wrapper)this.handler.getDevice()).getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletOLED128x64V2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOLED128x64V2Actions) {
            return ((BrickletOLED128x64V2Actions) actions).brickletOLED128x64V2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED128x64V2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletOLED128x64V2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletOLED128x64V2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletOLED128x64V2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOLED128x64V2Actions) {
            return ((BrickletOLED128x64V2Actions) actions).brickletOLED128x64V2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED128x64V2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletOLED128x64V2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletOLED128x64V2Wrapper)this.handler.getDevice()).getBootloaderMode();
        
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletOLED128x64V2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOLED128x64V2Actions) {
            return ((BrickletOLED128x64V2Actions) actions).brickletOLED128x64V2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED128x64V2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletOLED128x64V2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletOLED128x64V2.SPITFPErrorCount value = ((BrickletOLED128x64V2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletOLED128x64V2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOLED128x64V2Actions) {
            return ((BrickletOLED128x64V2Actions) actions).brickletOLED128x64V2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED128x64V2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletOLED128x64V2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletOLED128x64V2.Identity value = ((BrickletOLED128x64V2Wrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletOLED128x64V2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOLED128x64V2Actions) {
            return ((BrickletOLED128x64V2Actions) actions).brickletOLED128x64V2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED128x64V2Actions class.");
        }
    }
}
