package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletOLED64x48;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletOLED64x48Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Write")
    public void brickletOLED64x48Write(
            @ActionInput(name = "data") short[] data) throws TinkerforgeException {
        ((BrickletOLED64x48Wrapper)this.handler.getDevice()).write(data);
        
    }

    public static void brickletOLED64x48Write(@Nullable ThingActions actions, short[] data) throws TinkerforgeException {
        if (actions instanceof BrickletOLED64x48Actions) {
            ((BrickletOLED64x48Actions) actions).brickletOLED64x48Write(data);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED64x48Actions class.");
        }
    }

    @RuleAction(label = "New Window")
    public void brickletOLED64x48NewWindow(
            @ActionInput(name = "columnFrom") short columnFrom,
            @ActionInput(name = "columnTo") short columnTo,
            @ActionInput(name = "rowFrom") short rowFrom,
            @ActionInput(name = "rowTo") short rowTo) throws TinkerforgeException {
        ((BrickletOLED64x48Wrapper)this.handler.getDevice()).newWindow(columnFrom, columnTo, rowFrom, rowTo);
        
    }

    public static void brickletOLED64x48NewWindow(@Nullable ThingActions actions, short columnFrom, short columnTo, short rowFrom, short rowTo) throws TinkerforgeException {
        if (actions instanceof BrickletOLED64x48Actions) {
            ((BrickletOLED64x48Actions) actions).brickletOLED64x48NewWindow(columnFrom, columnTo, rowFrom, rowTo);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED64x48Actions class.");
        }
    }

    @RuleAction(label = "Clear Display")
    public void brickletOLED64x48ClearDisplay(
            ) throws TinkerforgeException {
        ((BrickletOLED64x48Wrapper)this.handler.getDevice()).clearDisplay();
        
    }

    public static void brickletOLED64x48ClearDisplay(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOLED64x48Actions) {
            ((BrickletOLED64x48Actions) actions).brickletOLED64x48ClearDisplay();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED64x48Actions class.");
        }
    }

    @RuleAction(label = "Write Line")
    public void brickletOLED64x48WriteLine(
            @ActionInput(name = "line") short line,
            @ActionInput(name = "position") short position,
            @ActionInput(name = "text") String text) throws TinkerforgeException {
        ((BrickletOLED64x48Wrapper)this.handler.getDevice()).writeLine(line, position, text);
        
    }

    public static void brickletOLED64x48WriteLine(@Nullable ThingActions actions, short line, short position, String text) throws TinkerforgeException {
        if (actions instanceof BrickletOLED64x48Actions) {
            ((BrickletOLED64x48Actions) actions).brickletOLED64x48WriteLine(line, position, text);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED64x48Actions class.");
        }
    }

    @RuleAction(label = "Get Display Configuration")
    public @ActionOutput(name = "contrast", type="short")
           @ActionOutput(name = "invert", type="boolean")
           Map<String, Object> brickletOLED64x48GetDisplayConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletOLED64x48.DisplayConfiguration value = ((BrickletOLED64x48Wrapper)this.handler.getDevice()).getDisplayConfiguration();
        
        result.put("contrast", value.contrast);
        result.put("invert", value.invert);
        return result;
    }

    public static Map<String, Object> brickletOLED64x48GetDisplayConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOLED64x48Actions) {
            return ((BrickletOLED64x48Actions) actions).brickletOLED64x48GetDisplayConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED64x48Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletOLED64x48GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletOLED64x48.Identity value = ((BrickletOLED64x48Wrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletOLED64x48GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOLED64x48Actions) {
            return ((BrickletOLED64x48Actions) actions).brickletOLED64x48GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED64x48Actions class.");
        }
    }
}
