package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletPTCV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletPTCV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletPTCV2GetTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletPTCV2Wrapper)this.handler.getDevice()).getTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPTCV2Actions) {
            return ((BrickletPTCV2Actions) actions).brickletPTCV2GetTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCV2Actions class.");
        }
    }

    @RuleAction(label = "Is Sensor Connected")
    public @ActionOutput(name = "connected", type="boolean")
           Map<String, Object> brickletPTCV2IsSensorConnected(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickletPTCV2Wrapper)this.handler.getDevice()).isSensorConnected();
        
        result.put("connected", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2IsSensorConnected(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPTCV2Actions) {
            return ((BrickletPTCV2Actions) actions).brickletPTCV2IsSensorConnected();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCV2Actions class.");
        }
    }

    @RuleAction(label = "Get Wire Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletPTCV2GetWireMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletPTCV2Wrapper)this.handler.getDevice()).getWireMode();
        
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetWireMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPTCV2Actions) {
            return ((BrickletPTCV2Actions) actions).brickletPTCV2GetWireMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCV2Actions class.");
        }
    }

    @RuleAction(label = "Get Resistance")
    public @ActionOutput(name = "resistance", type="int")
           Map<String, Object> brickletPTCV2GetResistance(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletPTCV2Wrapper)this.handler.getDevice()).getResistance();
        
        result.put("resistance", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetResistance(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPTCV2Actions) {
            return ((BrickletPTCV2Actions) actions).brickletPTCV2GetResistance();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCV2Actions class.");
        }
    }

    @RuleAction(label = "Get Noise Rejection Filter")
    public @ActionOutput(name = "filter", type="int")
           Map<String, Object> brickletPTCV2GetNoiseRejectionFilter(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletPTCV2Wrapper)this.handler.getDevice()).getNoiseRejectionFilter();
        
        result.put("filter", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetNoiseRejectionFilter(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPTCV2Actions) {
            return ((BrickletPTCV2Actions) actions).brickletPTCV2GetNoiseRejectionFilter();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCV2Actions class.");
        }
    }

    @RuleAction(label = "Get Moving Average Configuration")
    public @ActionOutput(name = "movingAverageLengthResistance", type="int")
           @ActionOutput(name = "movingAverageLengthTemperature", type="int")
           Map<String, Object> brickletPTCV2GetMovingAverageConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletPTCV2.MovingAverageConfiguration value = ((BrickletPTCV2Wrapper)this.handler.getDevice()).getMovingAverageConfiguration();
        
        result.put("movingAverageLengthResistance", value.movingAverageLengthResistance);
        result.put("movingAverageLengthTemperature", value.movingAverageLengthTemperature);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetMovingAverageConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPTCV2Actions) {
            return ((BrickletPTCV2Actions) actions).brickletPTCV2GetMovingAverageConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletPTCV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletPTCV2Wrapper)this.handler.getDevice()).readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPTCV2Actions) {
            return ((BrickletPTCV2Actions) actions).brickletPTCV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletPTCV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletPTCV2Wrapper)this.handler.getDevice()).getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPTCV2Actions) {
            return ((BrickletPTCV2Actions) actions).brickletPTCV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletPTCV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletPTCV2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPTCV2Actions) {
            return ((BrickletPTCV2Actions) actions).brickletPTCV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletPTCV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletPTCV2Wrapper)this.handler.getDevice()).getBootloaderMode();
        
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPTCV2Actions) {
            return ((BrickletPTCV2Actions) actions).brickletPTCV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletPTCV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletPTCV2.SPITFPErrorCount value = ((BrickletPTCV2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPTCV2Actions) {
            return ((BrickletPTCV2Actions) actions).brickletPTCV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletPTCV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletPTCV2.Identity value = ((BrickletPTCV2Wrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPTCV2Actions) {
            return ((BrickletPTCV2Actions) actions).brickletPTCV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPTCV2Actions class.");
        }
    }
}
