package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletPiezoSpeaker;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletPiezoSpeakerActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Beep")
    public void brickletPiezoSpeakerBeep(
            @ActionInput(name = "duration") long duration,
            @ActionInput(name = "frequency") int frequency) throws TinkerforgeException {
        ((BrickletPiezoSpeakerWrapper)this.handler.getDevice()).beep(duration, frequency);
        
    }

    public static void brickletPiezoSpeakerBeep(@Nullable ThingActions actions, long duration, int frequency) throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerActions) {
            ((BrickletPiezoSpeakerActions) actions).brickletPiezoSpeakerBeep(duration, frequency);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerActions class.");
        }
    }

    @RuleAction(label = "Morse Code")
    public void brickletPiezoSpeakerMorseCode(
            @ActionInput(name = "morse") String morse,
            @ActionInput(name = "frequency") int frequency) throws TinkerforgeException {
        ((BrickletPiezoSpeakerWrapper)this.handler.getDevice()).morseCode(morse, frequency);
        
    }

    public static void brickletPiezoSpeakerMorseCode(@Nullable ThingActions actions, String morse, int frequency) throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerActions) {
            ((BrickletPiezoSpeakerActions) actions).brickletPiezoSpeakerMorseCode(morse, frequency);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletPiezoSpeakerGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletPiezoSpeaker.Identity value = ((BrickletPiezoSpeakerWrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletPiezoSpeakerGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletPiezoSpeakerActions) {
            return ((BrickletPiezoSpeakerActions) actions).brickletPiezoSpeakerGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletPiezoSpeakerActions class.");
        }
    }
}
