package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletRS232V2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletRS232V2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Write")
    public @ActionOutput(name = "messageWritten", type="int")
           Map<String, Object> brickletRS232V2Write(
            @ActionInput(name = "message") char[] message) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRS232V2Wrapper)this.handler.getDevice()).write(message);
        
        result.put("messageWritten", value);
        return result;
    }

    public static Map<String, Object> brickletRS232V2Write(@Nullable ThingActions actions, char[] message) throws TinkerforgeException {
        if (actions instanceof BrickletRS232V2Actions) {
            return ((BrickletRS232V2Actions) actions).brickletRS232V2Write(message);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRS232V2Actions class.");
        }
    }

    @RuleAction(label = "Read")
    public @ActionOutput(name = "message", type="char[]")
           Map<String, Object> brickletRS232V2Read(
            @ActionInput(name = "length") int length) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        char[] value = ((BrickletRS232V2Wrapper)this.handler.getDevice()).read(length);
        
        result.put("message", value);
        return result;
    }

    public static Map<String, Object> brickletRS232V2Read(@Nullable ThingActions actions, int length) throws TinkerforgeException {
        if (actions instanceof BrickletRS232V2Actions) {
            return ((BrickletRS232V2Actions) actions).brickletRS232V2Read(length);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRS232V2Actions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "baudrate", type="long")
           @ActionOutput(name = "parity", type="int")
           @ActionOutput(name = "stopbits", type="int")
           @ActionOutput(name = "wordlength", type="int")
           @ActionOutput(name = "flowcontrol", type="int")
           Map<String, Object> brickletRS232V2GetConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRS232V2.Configuration value = ((BrickletRS232V2Wrapper)this.handler.getDevice()).getConfiguration();
        
        result.put("baudrate", value.baudrate);
        result.put("parity", value.parity);
        result.put("stopbits", value.stopbits);
        result.put("wordlength", value.wordlength);
        result.put("flowcontrol", value.flowcontrol);
        return result;
    }

    public static Map<String, Object> brickletRS232V2GetConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRS232V2Actions) {
            return ((BrickletRS232V2Actions) actions).brickletRS232V2GetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRS232V2Actions class.");
        }
    }

    @RuleAction(label = "Get Buffer Config")
    public @ActionOutput(name = "sendBufferSize", type="int")
           @ActionOutput(name = "receiveBufferSize", type="int")
           Map<String, Object> brickletRS232V2GetBufferConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRS232V2.BufferConfig value = ((BrickletRS232V2Wrapper)this.handler.getDevice()).getBufferConfig();
        
        result.put("sendBufferSize", value.sendBufferSize);
        result.put("receiveBufferSize", value.receiveBufferSize);
        return result;
    }

    public static Map<String, Object> brickletRS232V2GetBufferConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRS232V2Actions) {
            return ((BrickletRS232V2Actions) actions).brickletRS232V2GetBufferConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRS232V2Actions class.");
        }
    }

    @RuleAction(label = "Get Buffer Status")
    public @ActionOutput(name = "sendBufferUsed", type="int")
           @ActionOutput(name = "receiveBufferUsed", type="int")
           Map<String, Object> brickletRS232V2GetBufferStatus(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRS232V2.BufferStatus value = ((BrickletRS232V2Wrapper)this.handler.getDevice()).getBufferStatus();
        
        result.put("sendBufferUsed", value.sendBufferUsed);
        result.put("receiveBufferUsed", value.receiveBufferUsed);
        return result;
    }

    public static Map<String, Object> brickletRS232V2GetBufferStatus(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRS232V2Actions) {
            return ((BrickletRS232V2Actions) actions).brickletRS232V2GetBufferStatus();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRS232V2Actions class.");
        }
    }

    @RuleAction(label = "Get Error Count")
    public @ActionOutput(name = "errorCountOverrun", type="long")
           @ActionOutput(name = "errorCountParity", type="long")
           Map<String, Object> brickletRS232V2GetErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRS232V2.ErrorCount value = ((BrickletRS232V2Wrapper)this.handler.getDevice()).getErrorCount();
        
        result.put("errorCountOverrun", value.errorCountOverrun);
        result.put("errorCountParity", value.errorCountParity);
        return result;
    }

    public static Map<String, Object> brickletRS232V2GetErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRS232V2Actions) {
            return ((BrickletRS232V2Actions) actions).brickletRS232V2GetErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRS232V2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletRS232V2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletRS232V2Wrapper)this.handler.getDevice()).readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletRS232V2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRS232V2Actions) {
            return ((BrickletRS232V2Actions) actions).brickletRS232V2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRS232V2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletRS232V2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRS232V2Wrapper)this.handler.getDevice()).getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletRS232V2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRS232V2Actions) {
            return ((BrickletRS232V2Actions) actions).brickletRS232V2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRS232V2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletRS232V2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRS232V2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletRS232V2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRS232V2Actions) {
            return ((BrickletRS232V2Actions) actions).brickletRS232V2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRS232V2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletRS232V2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRS232V2Wrapper)this.handler.getDevice()).getBootloaderMode();
        
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletRS232V2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRS232V2Actions) {
            return ((BrickletRS232V2Actions) actions).brickletRS232V2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRS232V2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletRS232V2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRS232V2.SPITFPErrorCount value = ((BrickletRS232V2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletRS232V2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRS232V2Actions) {
            return ((BrickletRS232V2Actions) actions).brickletRS232V2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRS232V2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletRS232V2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRS232V2.Identity value = ((BrickletRS232V2Wrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletRS232V2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRS232V2Actions) {
            return ((BrickletRS232V2Actions) actions).brickletRS232V2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRS232V2Actions class.");
        }
    }
}
