package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletSegmentDisplay4x7;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletSegmentDisplay4x7Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Set Segments")
    public void brickletSegmentDisplay4x7SetSegments(
            @ActionInput(name = "segments") short[] segments,
            @ActionInput(name = "brightness") short brightness,
            @ActionInput(name = "colon") boolean colon) throws TinkerforgeException {
        ((BrickletSegmentDisplay4x7Wrapper)this.handler.getDevice()).setSegments(segments, brightness, colon);
        if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletSegmentDisplay4x7Segments")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletSegmentDisplay4x7Segments"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletSegmentDisplay4x7Colon")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletSegmentDisplay4x7Colon"), RefreshType.REFRESH);
}
if(handler.getThing().getChannel(new ChannelUID(handler.getThing().getUID(), "BrickletSegmentDisplay4x7Brightness")) != null){
	this.handler.handleCommand(new ChannelUID(handler.getThing().getUID(), "BrickletSegmentDisplay4x7Brightness"), RefreshType.REFRESH);
}
    }

    public static void brickletSegmentDisplay4x7SetSegments(@Nullable ThingActions actions, short[] segments, short brightness, boolean colon) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7Actions) {
            ((BrickletSegmentDisplay4x7Actions) actions).brickletSegmentDisplay4x7SetSegments(segments, brightness, colon);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7Actions class.");
        }
    }

    @RuleAction(label = "Get Segments")
    public @ActionOutput(name = "segments", type="short[]")
           @ActionOutput(name = "brightness", type="short")
           @ActionOutput(name = "colon", type="boolean")
           Map<String, Object> brickletSegmentDisplay4x7GetSegments(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletSegmentDisplay4x7.Segments value = ((BrickletSegmentDisplay4x7Wrapper)this.handler.getDevice()).getSegments();
        
        result.put("segments", value.segments);
        result.put("brightness", value.brightness);
        result.put("colon", value.colon);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7GetSegments(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7Actions) {
            return ((BrickletSegmentDisplay4x7Actions) actions).brickletSegmentDisplay4x7GetSegments();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7Actions class.");
        }
    }

    @RuleAction(label = "Start Counter")
    public void brickletSegmentDisplay4x7StartCounter(
            @ActionInput(name = "valueFrom") short valueFrom,
            @ActionInput(name = "valueTo") short valueTo,
            @ActionInput(name = "increment") short increment,
            @ActionInput(name = "length") long length) throws TinkerforgeException {
        ((BrickletSegmentDisplay4x7Wrapper)this.handler.getDevice()).startCounter(valueFrom, valueTo, increment, length);
        
    }

    public static void brickletSegmentDisplay4x7StartCounter(@Nullable ThingActions actions, short valueFrom, short valueTo, short increment, long length) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7Actions) {
            ((BrickletSegmentDisplay4x7Actions) actions).brickletSegmentDisplay4x7StartCounter(valueFrom, valueTo, increment, length);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7Actions class.");
        }
    }

    @RuleAction(label = "Get Counter Value")
    public @ActionOutput(name = "value", type="int")
           Map<String, Object> brickletSegmentDisplay4x7GetCounterValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletSegmentDisplay4x7Wrapper)this.handler.getDevice()).getCounterValue();
        
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7GetCounterValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7Actions) {
            return ((BrickletSegmentDisplay4x7Actions) actions).brickletSegmentDisplay4x7GetCounterValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletSegmentDisplay4x7GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletSegmentDisplay4x7.Identity value = ((BrickletSegmentDisplay4x7Wrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletSegmentDisplay4x7GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7Actions) {
            return ((BrickletSegmentDisplay4x7Actions) actions).brickletSegmentDisplay4x7GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7Actions class.");
        }
    }
}
