package org.eclipse.smarthome.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletVoltageCurrentV2;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletVoltageCurrentV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickletVoltageCurrentV2GetVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletVoltageCurrentV2Wrapper)this.handler.getDevice()).getVoltage();
        
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).brickletVoltageCurrentV2GetVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }

    @RuleAction(label = "Get Current")
    public @ActionOutput(name = "current", type="int")
           Map<String, Object> brickletVoltageCurrentV2GetCurrent(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletVoltageCurrentV2Wrapper)this.handler.getDevice()).getCurrent();
        
        result.put("current", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetCurrent(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).brickletVoltageCurrentV2GetCurrent();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }

    @RuleAction(label = "Get Power")
    public @ActionOutput(name = "power", type="int")
           Map<String, Object> brickletVoltageCurrentV2GetPower(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletVoltageCurrentV2Wrapper)this.handler.getDevice()).getPower();
        
        result.put("power", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetPower(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).brickletVoltageCurrentV2GetPower();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "averaging", type="int")
           @ActionOutput(name = "voltageConversionTime", type="int")
           @ActionOutput(name = "currentConversionTime", type="int")
           Map<String, Object> brickletVoltageCurrentV2GetConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletVoltageCurrentV2.Configuration value = ((BrickletVoltageCurrentV2Wrapper)this.handler.getDevice()).getConfiguration();
        
        result.put("averaging", value.averaging);
        result.put("voltageConversionTime", value.voltageConversionTime);
        result.put("currentConversionTime", value.currentConversionTime);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).brickletVoltageCurrentV2GetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }

    @RuleAction(label = "Get Calibration")
    public @ActionOutput(name = "voltageMultiplier", type="int")
           @ActionOutput(name = "voltageDivisor", type="int")
           @ActionOutput(name = "currentMultiplier", type="int")
           @ActionOutput(name = "currentDivisor", type="int")
           Map<String, Object> brickletVoltageCurrentV2GetCalibration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletVoltageCurrentV2.Calibration value = ((BrickletVoltageCurrentV2Wrapper)this.handler.getDevice()).getCalibration();
        
        result.put("voltageMultiplier", value.voltageMultiplier);
        result.put("voltageDivisor", value.voltageDivisor);
        result.put("currentMultiplier", value.currentMultiplier);
        result.put("currentDivisor", value.currentDivisor);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetCalibration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).brickletVoltageCurrentV2GetCalibration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletVoltageCurrentV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletVoltageCurrentV2Wrapper)this.handler.getDevice()).readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).brickletVoltageCurrentV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletVoltageCurrentV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletVoltageCurrentV2Wrapper)this.handler.getDevice()).getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).brickletVoltageCurrentV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletVoltageCurrentV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletVoltageCurrentV2Wrapper)this.handler.getDevice()).getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).brickletVoltageCurrentV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletVoltageCurrentV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletVoltageCurrentV2Wrapper)this.handler.getDevice()).getBootloaderMode();
        
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).brickletVoltageCurrentV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletVoltageCurrentV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletVoltageCurrentV2.SPITFPErrorCount value = ((BrickletVoltageCurrentV2Wrapper)this.handler.getDevice()).getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).brickletVoltageCurrentV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletVoltageCurrentV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletVoltageCurrentV2.Identity value = ((BrickletVoltageCurrentV2Wrapper)this.handler.getDevice()).getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).brickletVoltageCurrentV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }
}
